<?php
/**
 * The Juno script to create a new book or edit an existing book.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/books/edit.php,v 1.20 2005/01/03 14:35:09 jan Exp $
 *
 * @todo Clean up and add book editing.
 */

@define('JUNO_BASE', dirname(__FILE__) . '/..');
require_once JUNO_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Variables.php';

/* Only admin should be using this. */
if (!Auth::getAuth()) {
    Horde::authenticationFailureRedirect();
}

$vars = &Variables::getDefaultVariables();
$book_id = $vars->get('id');

/* Set the correct title. */
if (!$book_id) {
    $title = _("New Accounting Book");
} else {
    $title = _("Edit Accounting Book");
}

/* Set the form object. */
$form = &Horde_Form::singleton('', $vars, $title);

/* Check if a form is being edited. */
if ($book_id && !$form->isSubmitted()) {
    $editbook = &$books->getBook($book_id);
    $vars = &new Variables($editbook->getBookArray());
}

$books_list = array(DATATREE_ROOT => _("-- Top Level --"));

/* Set up the form. */
$form->setButtons(($book_id ? _("Modify") : _("Create")), true);
$form->addHidden('', 'id', 'int', false);
$form->addVariable(_("Basic configuration"), '', 'header', false);
$form->addVariable(_("Book"), 'name', 'text', true, false);
$form->addVariable(_("Start date"), 'start_date', 'monthdayyear', true, false, null, array(date('Y'), date('Y') - 5, true, 3));

/* The list should be automatically generated from NLS? */
$v = &$form->addVariable(_("Main currency"), 'currency', 'enum', true, false, null, array(Juno::getCurrencies(), true));

$form->addVariable(_("Advanced settings"), '', 'header', false);
$form->addVariable(_("Sub book for"), 'parent_id', 'enum', false, false, null, array($books_list));
$form->addVariable(_("Multiple currencies"), 'multicurrency', 'boolean', false, false, _("Select this if you plan to have entries in currencies other than your main currency"));


if ($form->validate($vars)) {
    $form->getInfo($vars, $info);
    $book_id = $books->saveBook($info);
    if (is_a($book_id, 'PEAR_Error')) {
        Horde::logMessage($book_id, __FILE__, __LINE__, LOG_ERR);
        $notification->push(sprintf(_("Error saving book. %s."), $book_id->getMessage()), 'horde.error');
    } else {
        $notification->push(_("Book details saved."), 'horde.success');
        $url = Horde::applicationUrl('books/index.php', true);
        header('Location: ' . $url);
        exit;
    }
}

/* Render the form. */
require_once 'Horde/Form/Renderer.php';
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'edit.php', 'post');

$template = &new Horde_Template();
$template->set('menu', Juno::getMenu('string'));
$template->set('main', $main);
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));

require JUNO_TEMPLATES . '/common-header.inc';
echo $template->fetch(JUNO_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
