<?php
/**
 * The Juno script to create a new book or edit an existing book.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/invoices/edit.php,v 1.5 2005/01/03 14:35:10 jan Exp $
 *
 * @todo Clean up and add book editing.
 */

@define('JUNO_BASE', dirname(__FILE__) . '/..');
require_once JUNO_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Variables.php';

/* Only admin should be using this. */
if (!Auth::getAuth()) {
    Horde::authenticationFailureRedirect();
}

$vars = &Variables::getDefaultVariables();
$transaction_id = $vars->get('id');

/* Set the correct title. */
if (!$transaction_id) {
    $title = _("New Invoice");
} else {
    $title = _("Edit Invoice");
}

/* Set the form object. */
$form = &Horde_Form::singleton('', $vars, $title);

/* Check if a form is being edited. */
if ($transaction_id && !$form->isSubmitted()) {
    $editbook = &$books->getBook($transaction_id);
    $vars = &new Variables($editbook->getBookArray());
}

$books_list = array(DATATREE_ROOT => _("-- Top Level --"));

/* Set up the form. */
$form->setButtons(($transaction_id ? _("Modify") : _("Create")), true);
Juno::setTransactionFields($form);

/* Add terms selection. */
require_once 'Horde/Form/Action.php';
$action = &Horde_Form_Action::factory('submit');
$v = &$form->addVariable(_("Terms"), 'attr[terms]', 'enum', false, false, null, array(Juno::getTerms(), true));
$v->setAction($action);
$v->setOption('trackchange', true);
if ($vars->get('terms') == 'custom') {
    $form->addVariable(_("Due date"), 'attr[duedate]', 'monthdayyear', true, false, null, array(date('Y') + 2, date('Y') - 5, true, 3));
}

if ($form->validate($vars)) {
    $form->getInfo($vars, $info);
    $transaction_id = $juno->saveTransaction($info);
    if (is_a($transaction_id, 'PEAR_Error')) {
        Horde::logMessage($transaction_id, __FILE__, __LINE__, LOG_ERR);
        $notification->push(sprintf(_("Error saving invoice. %s."), $transaction_id->getMessage()), 'horde.error');
    } else {
        $notification->push(_("Invoice details saved."), 'horde.success');
        $url = Horde::applicationUrl('invoices/index.php', true);
        header('Location: ' . $url);
        exit;
    }
}

/* Render the form. */
require_once 'Horde/Form/Renderer.php';
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'edit.php', 'post');

$template = &new Horde_Template();
$template->set('menu', Juno::getMenu('string'));
$template->set('main', $main);
$template->set('book_status', $book_status, true);
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));

require JUNO_TEMPLATES . '/common-header.inc';
echo $template->fetch(JUNO_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
