<?php

require_once 'Horde/SQL/Attributes.php';

/**
 * Juno storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql, etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The SQL tables are defined in juno/scripts/drivers/juno.sql.
 *
 * $Horde: juno/lib/Driver/sql.php,v 1.11 2004/10/08 15:45:23 chuck Exp $
 *
 * @version $Revision: 1.11 $
 * @package Juno
 */
class Juno_Driver_sql extends Juno_Driver {

    /** Handle for the current database connection.
        @var object DB $db */
    var $_db;

    var $_transaction_attr;

    /**
     * Constructs a new SQL storage object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Juno_Driver_sql($params)
    {
        parent::Juno_Driver($params);
        $this->initialise();

        $attr_params = array ('primary_table'   => 'juno_transactions',
                              'attribute_table' => 'juno_transaction_attributes',
                              'id_column'       => 'item_id');
        $this->_transaction_attr = &Horde_SQL_Attributes::singleton($this->_db, $attr_params);

    }

    function saveTransaction($info)
    {
        global $book;

        if (empty($info['transaction_id'])) {
            $info['transaction_id'] = $this->_db->nextId('juno_transactions');
            $sql = 'INSERT INTO juno_transactions (book_id, transaction_ref, transaction_currency, transaction_desc, transaction_enterdate, transaction_postdate, transaction_id) VALUES (%s, %s, %s, %s, %s, %s, %s)';
        } else {
            $sql = 'UPDATE juno_transactions SET book_id = %s, transaction_ref = %s, transaction_currency = %s, transaction_desc = %s, transaction_enterdate = %s, transaction_postdate = %s WHERE transaction_id = %s';
        }
        $sql = sprintf($sql,
                       $book->getId(),
                       $this->_db->quote($info['ref']),
                       (isset($info['currency']) ? $this->_db->quote($info['currency']) : $this->_db->quote($book->getCurrency())),
                       $this->_db->quote($info['desc']),
                       $this->_db->quote($info['enterdate']),
                       $this->_db->quote($info['postdate']),
                       (int)$info['transaction_id']);
        Horde::logMessage('SQL Query by Juno_Driver_sql::saveTransaction(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $info['transaction_id'];
    }

    function initialise()
    {
        global $registry;

        Horde::assertDriverConfig($this->_params, 'sql',
            array('phptype', 'hostspec', 'username', 'database'));

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        return true;
    }

}
