<?php
/**
 * Juno_Stocks_nasdaq
 * 
 * Copyright (c) 2002 Eric Rechlin <eric@hpcalc.org>.
 * Copyright (c) 2000-2004 Charles Hagenbuch <chuck@horde.org>.
 *
 * $Horde: juno/lib/Stocks/nasdaq.php,v 1.12 2004/01/21 17:17:08 chuck Exp $
 *
 * @author  Eric Rechlin <eric@hpcalc.org>
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.12 $
 * @package Juno
 */
class Juno_Stocks_nasdaq extends Juno_Stocks {

    /**
     * Array of capabilities that this backend supports.
     * @var array $_capabilities
     */
    var $_capabilities = array('quotes' => true,
                               'charts' => false);

    /**
     * The XML parser.
     * @var resource $_parser
     */
    var $_parser = null;

    /**
     * The array of quotes that we've fetched.
     * @var array $_quotes
     */
    var $_quotes = array();

    /**
     * Current XML key string.
     * @var string $_keyStack
     */
    var $_keyStack = '';

    /**
     * Pointer to the current quote.
     * @var array $_current
     */
    var $_current = null;

    /**
     * The base URL for Nasdaq quotes.
     * @var string $_baseUrl
     */
    var $_baseUrl = 'http://quotes.nasdaq.com/quote.dll?page=xml&mode=stock&symbol=';

    /**
     * Keys that tell us where we are in the XML document.
     * @var array $_keys
     */
    var $_keys = array('nameKey' => '^nasdaqamex-dot-com^equity-quote^issue-name',
                       'marketCenterKey' => '^nasdaqamex-dot-com^equity-quote^market-center-code',
                       'marketStatusKey' => '^nasdaqamex-dot-com^equity-quote^market-status',
                       'typeKey' => '^nasdaqamex-dot-com^equity-quote^issue-type-code',
                       'todaysHighKey' => '^nasdaqamex-dot-com^equity-quote^todays-high-price',
                       'todaysLowKey' => '^nasdaqamex-dot-com^equity-quote^todays-low-price',
                       'yearHighKey' => '^nasdaqamex-dot-com^equity-quote^fifty-two-wk-high-price',
                       'yearLowKey' => '^nasdaqamex-dot-com^equity-quote^fifty-two-wk-low-price',
                       'lastPriceKey' => '^nasdaqamex-dot-com^equity-quote^last-sale-price',
                       'changePriceKey' => '^nasdaqamex-dot-com^equity-quote^net-change-price',
                       'changePctKey' => '^nasdaqamex-dot-com^equity-quote^net-change-pct',
                       'volumeKey' => '^nasdaqamex-dot-com^equity-quote^share-volume-qty',
                       'lastCloseKey' => '^nasdaqamex-dot-com^equity-quote^previous-close-price',
                       'bestBidKey' => '^nasdaqamex-dot-com^equity-quote^best-bid-price',
                       'bestAskKey' => '^nasdaqamex-dot-com^equity-quote^best-ask-price',
                       'peRatioKey' => '^nasdaqamex-dot-com^equity-quote^current-pe-ratio',
                       'totalSharesKey' => '^nasdaqamex-dot-com^equity-quote^total-outstanding-shares-qty',
                       'yieldPctKey' => '^nasdaqamex-dot-com^equity-quote^current-yield-pct',
                       'earningsKey' => '^nasdaqamex-dot-com^equity-quote^earnings-actual-eps-amt',
                       'dividendAmtKey' => '^nasdaqamex-dot-com^equity-quote^cash-dividend-amt',
                       'dividendDateKey' => '^nasdaqamex-dot-com^equity-quote^cash-dividend-ex-date',
                       'sp500BetaKey' => '^nasdaqamex-dot-com^equity-quote^sp500-beta-num',
                       'tradeDateKey' => '^nasdaqamex-dot-com^equity-quote^trade-datetime',
                       'address1Key' => '^nasdaqamex-dot-com^equity-quote^issuer-address-line1-txt',
                       'cityStateKey' => '^nasdaqamex-dot-com^equity-quote^issuer-city-state-zip-txt',
                       'phoneKey' => '^nasdaqamex-dot-com^equity-quote^issuer-phone-num',
                       'webSiteKey' => '^nasdaqamex-dot-com^equity-quote^issuer-web-site-url',
                       'logoKey' => '^nasdaqamex-dot-com^equity-quote^issuer-logo-url',
                       'tradingStatusKey' => '^nasdaqamex-dot-com^equity-quote^trading-status',
                       'marketCapKey' => '^nasdaqamex-dot-com^equity-quote^market-capitalization-amt');

    function _getQuotes($symbols)
    {
        $url = $this->_baseUrl . implode('&symbol=', $symbols);
        $data = Juno::_readUrl($url);

        $this->_initialize();

        if (!$this->_parser) {
            return PEAR::raiseError(_("Could not find XML parser handle."));
        }

        if (!xml_parse($this->_parser, $data)) {
            return PEAR::raiseError(sprintf(_("XML error: %s at line %d"), xml_error_string(xml_get_error_code($this->_parser)), xml_get_current_line_number($this->_parser)));
        }

        xml_parser_free($this->_parser);

        return $this->_quotes;
    }

    /**
     * Initialize the XML parser.
     */
    function _initialize()
    {
        // Initialize some variables.
        $this->_quotes = array();
        $this->_keyStack = '';
        $this->_current = null;

        // Create the XML parser.
        $this->_parser = xml_parser_create();
        xml_set_object($this->_parser, $this);
        xml_parser_set_option($this->_parser, XML_OPTION_CASE_FOLDING, false);
        xml_set_element_handler($this->_parser, '_start', '_end');
        xml_set_character_data_handler($this->_parser, '_data');
        xml_set_default_handler($this->_parser, '_default');
    }

    /**
     * Start collecting data about a new element.
     *
     * @access private
     */
    function _start($parser, $name, $attributes)
    {
        $this->_keyStack .= "^$name";

        if ($name == 'equity-quote') {
            $symbol = String::lower($attributes['symbol']);
            if (!isset($this->_quotes[$symbol])) {
                $this->_quotes[$symbol] = array();
            }
            $this->_current = &$this->_quotes[$symbol];
        }
    }

    /**
     * Handle the ends of XML elements - wrap up whatever we've been
     * putting together and store it for safekeeping.
     *
     * @access private
     */
    function _end($parser, $name)
    {
        $currentPosition = strrpos($this->_keyStack, '^');
        $this->_keyStack = substr($this->_keyStack, 0, $currentPosition);
    }

    /**
     * The handler for character data encountered in the XML file.
     *
     * @access private
     */
    function _data($parser, $data)
    {
        $data = trim($data);
        switch ($this->_keyStack) {
        case $this->_keys['nameKey']:
            $this->_current['name'] = $data;
            break;

        case $this->_keys['marketStatusKey']:
            $this->_current['marketStatus'] = $data;
            break;

        case $this->_keys['marketCenterKey']:
            $this->_current['marketCenter'] = $data;
            break;

        case $this->_keys['typeKey']:
            $this->_current['type'] = $data;
            break;

        case $this->_keys['todaysHighKey']:
            $this->_current['high'] = $data;
            break;

        case $this->_keys['todaysLowKey']:
            $this->_current['low'] = $data;
            break;

        case $this->_keys['yearHighKey']:
            $this->_current['yearlyHigh'] = sprintf('%.2f', $data);
            break;
        case $this->_keys['yearLowKey']:
            $this->_current['yearlyLow'] = sprintf('%.2f', $data);
            break;

        case $this->_keys['lastPriceKey']:
            $this->_current['lastPrice'] = sprintf('%.2f', $data);
            break;

        case $this->_keys['changePriceKey']:
            $this->_current['changePrice'] = $data;
            break;

        case $this->_keys['changePctKey']:
            $this->_current['changePct'] = $data;
            break;

        case $this->_keys['volumeKey']:
            $this->_current['volume'] = $data;
            break;

        case $this->_keys['lastCloseKey']:
            $this->_current['lastClose'] = sprintf('%.2f', $data);
            break;

        case $this->_keys['bestBidKey']:
            $this->_current['bestBid'] = $data;
            break;

        case $this->_keys['bestAskKey']:
            $this->_current['bestAsk'] = $data;
            break;

        case $this->_keys['peRatioKey']:
            $this->_current['peRatio'] = $data;
            break;

        case $this->_keys['totalSharesKey']:
            $this->_current['totalShares'] = $data;
            break;

        case $this->_keys['yieldPctKey']:
            $this->_current['yieldPct'] = $data;
            break;

        case $this->_keys['earningsKey']:
            $this->_current['earnings'] = $data;
            break;

        case $this->_keys['dividendAmtKey']:
            $this->_current['dividendAmt'] = $data;
            break;

        case $this->_keys['dividendDateKey']:
            $this->_current['dividendDate'] = $data;
            break;

        case $this->_keys['sp500BetaKey']:
            $this->_current['sp500Beta'] = $data;
            break;

        case $this->_keys['tradeDateKey']:
            $this->_current['tradeDate'] = $data;
            break;

        case $this->_keys['address1Key']:
            $this->_current['address1'] = $data;
            break;

        case $this->_keys['cityStateKey']:
            $this->_current['cityState'] = $data;
            break;

        case $this->_keys['phoneKey']:
            $this->_current['phone'] = $data;
            break;

        case $this->_keys['webSiteKey']:
            $this->_current['website'] = $data;
            break;

        case $this->_keys['logoKey']:
            $this->_current['logo'] = $data;
            break;

        case $this->_keys['tradingStatusKey']:
            $this->_current['tradingStatus'] = $data;
            break;

        case $this->_keys['marketCapKey']:
            $this->_current['marketCap'] = $data;
            break;
        }
    }

    /**
     * Handles things that we don't recognize. A no-op.
     *
     * @access private
     */
    function _default($parser, $data)
    {
    }

}
