<?php
/**
 * The Juno script to add or modify accounts in a book.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/lists/editaccount.php,v 1.19 2005/01/03 14:35:10 jan Exp $
 *
 * @todo Add editing of accounts, link accounts to opened book.
 */

@define('JUNO_BASE', dirname(__FILE__) . '/..');
require_once JUNO_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Form/Action.php';
require_once 'Horde/Variables.php';

$accounts = &Juno_Lists::singleton('accounts');

$classes = Juno::getAccountClasses();
$types = Juno::getAccountTypes();

$vars = &Variables::getDefaultVariables();
$account_id = $vars->get('id');
$formname = $vars->get('formname');
$class = $vars->get('class');
$old_class = $vars->get('old_class');

$changed_class = false;
if ($old_class != $class) {
    $changed_class = true;
}

$class_label = '';
if (!empty($class)) {
    $class_label = $classes[$class];
}

/* Set the current class to the old class var. */
$vars->set('old_class', $class);

$form = &Horde_Form::singleton('', $vars, $account_id ? sprintf(_("Edit %s Account"), $class_label) : sprintf(_("New %s Account"), $class_label));

/* Set up the form. */
$form->_reset = true;
$form->addHidden('', 'id', 'int', false);
$form->addHidden('', 'old_class', 'text', false);
$v = &$form->addVariable(_("Account class"), 'class', 'enum', true, false, null, array($classes));
$action = &Horde_Form_Action::factory('submit');
$v->setAction($action);

if (!empty($class)) {
    $accounts_list = $accounts->getList(DATATREE_ROOT, false);
    $form->addVariable(_("Type"), 'type', 'enum', true, false, null, array($types));
    $form->addVariable(_("Subaccount of"), 'parent_id', 'enum', false, false, null, array($accounts_list));
    $form->addVariable(_("Number"), 'number', 'int', true);
    $form->addVariable(_("Name"), 'name', 'text', true);
    $form->addVariable(_("Description"), 'description', 'text', false);
    $form->addVariable(_("Opening balance"), 'start_balance', 'text', false);

    /* TODO: set the min date to be the start of the current
     * accounting period.  Set to -3 years for now. Would be good to
     * default to start of book. */
    $v = &$form->addVariable(_("Start date"), 'start_date', 'monthdayyear', false, false, null, array(date('Y', time()), date('Y', time()) - 3));
    $v->setDefault(date('Y-m-d', time()));
    $form->setButtons(($account_id ? _("Modify") : _("Create")));
} else {
    $form->setButtons(_("Continue"));
}

if ($formname && !$changed_class) {
    if ($form->validate($vars)) {
        $form->getInfo($vars, $info);
        $account_id = $accounts->saveListEntry($info);
        if (is_a($account_id, 'PEAR_Error')) {
            Horde::logMessage($account_id, __FILE__, __LINE__, LOG_ERR);
            $notification->push(sprintf(_("Error saving account. %s."), $account_id->getMessage()), 'horde.error');
        } else {
            $notification->push(_("Account details saved."), 'horde.success');
            $url = Horde::applicationUrl('lists/accounts.php', true);
            header('Location: ' . $url);
            exit;
        }
    }
}

$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'editaccount.php', 'post');

$template = &new Horde_Template();
$template->set('menu', Juno::getMenu('string'));
$template->set('main', $main);
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));

require JUNO_TEMPLATES . '/common-header.inc';
echo $template->fetch(JUNO_BASE . '/templates/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
